<?php

/**
 * Copyright (c) D3 Data Development (Inh. Thomas Dartsch)
 *
 * For the full copyright and license information, please view
 * the LICENSE file that was distributed with this source code.
 *
 * https://www.d3data.de
 *
 * @copyright (C) D3 Data Development (Inh. Thomas Dartsch)
 * @author    D3 Data Development - Max Buhe, Daniel Seifert <info@shopmodule.com>
 * @link      https://www.oxidmodule.com
 */

declare(strict_types=1);

namespace D3\SensitiveMessageFormatter;

use GuzzleHttp\MessageFormatter;
use Psr\Http\Message\RequestInterface;
use Psr\Http\Message\ResponseInterface;
use Throwable;

class sensitiveMessageFormatter extends MessageFormatter
{
    /**
     * @param string|null $template
     * @param string[] $anonymizations
     * @param string|null $replacement
     */
    public function __construct(
        ?string $template = self::CLF,
        protected array $anonymizations = [],
        protected ?string $replacement = null
    ) {
        $this->replacement ??= '*****';
        $this->convertStringsToRegex($this->anonymizations);

        parent::__construct($template);
    }

    /**
     * @param string[] $search
     * @return void
     */
    protected function convertStringsToRegex(array $search = []): void
    {
        array_map(
            function ($search) use (&$searchStrings) {
                if (!$this->stringIsRegexp($search)) {
                    $searchStrings[] = '/'.preg_quote($search, '/').'/i';
                    if (urlencode($search) !== $search) {
                        $searchStrings[] = '/' . preg_quote(urlencode($search), '/') . '/i';
                    }
                } else {
                    $searchStrings[] = $search;
                }
            },
            $search
        );

        $this->anonymizations = $searchStrings;
    }

    public function format(
        RequestInterface $request,
        ResponseInterface $response = null,
        Throwable $error = null
    ): string {
        $result = parent::format($request, $response, $error);

        if (count($this->anonymizations)) {
            $result = preg_replace(
                $this->anonymizations,
                $this->replacement,
                $result
            );
        }

        return $result;
    }

    protected function stringIsRegexp(string $string): bool
    {
        return @preg_match($string, '') !== false;
    }
}
